Package: pkgmaker
Type: Package
Title: Development Utilities for R Packages
Version: 0.32.10
Date: 2023-05-03
Authors@R: person("Renaud", "Gaujoux", email = "renozao@protonmail.com", role = c("aut", "cre"))
Description: Provides some low-level utilities to use for package
    development. It currently provides managers for multiple package specific
    options and registries, vignette, unit test and bibtex related utilities.
    It serves as a base package for packages like 'NMF', 'RcppOctave', 'doRNG', and
    as an incubator package for other general purposes utilities, that will
    eventually be packaged separately.
    It is still under heavy development and changes in the interface(s) are
    more than likely to happen.
License: GPL (>= 2)
URL: https://renozao.github.io/pkgmaker/
BugReports: https://github.com/renozao/pkgmaker/issues
LazyLoad: yes
Encoding: UTF-8
Depends: R (>= 3.0.0), methods, stats, registry
Imports: tools, grDevices, utils, graphics, codetools, digest, stringr,
        xtable, withr, assertthat
Suggests: devtools (>= 0.8), roxygen2, RUnit, testthat, knitr,
        rmarkdown, markdown (>= 1.3), yaml, Biobase, datasets,
        rbibutils (>= 1.3)
Collate: 'bibtex.R' 'colors.R' 'data.R' 'namespace.R' 'utils.R'
        'devutils.R' 'files.R' 'graphics.R' 'options.R' 'logging.R'
        'unitTests.R' 'is.R' 'knitr.R' 'registry.R' 'package-extra.R'
        'package.R' 'packages.R' 'project.R' 'rd.R' 'user.R'
        'vignette.R' 'with.r'
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2023-05-03 08:01:53 UTC; renaud
Author: Renaud Gaujoux [aut, cre]
Maintainer: Renaud Gaujoux <renozao@protonmail.com>
Repository: CRAN
Date/Publication: 2023-05-03 11:30:08 UTC
Built: R 4.4.0; ; 2026-07-18 10:24:03 UTC; unix
