# Generated by roxygen2: do not edit by hand

S3method("$",ExposeAttribute)
S3method("$<-",ExposeAttribute)
S3method("[[",package_options)
S3method("[[<-",package_options)
S3method(.DollarNames,ExposeAttribute)
S3method(addnames,array)
S3method(addnames,default)
S3method(addnames,matrix)
S3method(addnames,vector)
S3method(format,package_metaregistry)
S3method(format,package_subregistry)
S3method(print,ExposeAttribute)
S3method(print,package_metaregistry)
S3method(print,package_options)
S3method(print,str_diff)
S3method(runVignette,default)
S3method(runVignette,rnw_knitr)
S3method(runVignette,rnw_sweave)
S3method(xtable,package_metaregistry)
export("attr_mode<-")
export(.silenceF)
export(CRAN)
export(ExposeAttribute)
export(R.CMD)
export(R.SHLIB)
export(R.exec)
export(RdSection2latex)
export(Rversion)
export(Sys.getenv_value)
export(addNamespaceExport)
export(add_lib)
export(addnames)
export(allFormals)
export(alphacol)
export(as.package_options)
export(as.rnw)
export(as_package)
export(askUser)
export(attr_mode)
export(cgetAnywhere)
export(charmap)
export(checkPlot)
export(checkWarning)
export(citecmd)
export(compactVignettes)
export(compile_src)
export(digest_function)
export(exitCheck)
export(expand_dots)
export(expand_list)
export(extractLocalFun)
export(factor2character)
export(file_extension)
export(find_devpackage)
export(getBiocMirror)
export(getBiocRepos)
export(getLoadingNamespace)
export(gfile)
export(hasArg2)
export(hasEnvar)
export(hasNames)
export(hasPackageRegistry)
export(hook_backspace)
export(hook_toggle)
export(hook_try)
export(inSweave)
export(install.extrapackages)
export(install.extras)
export(irequire)
export(is.dir)
export(is.file)
export(is.verbose)
export(isCHECK)
export(isCRAN_timing)
export(isCRANcheck)
export(isDevNamespace)
export(isInteger)
export(isLoadingNamespace)
export(isManualVignette)
export(isNamespaceLoaded2)
export(isNumber)
export(isPackageInstalled)
export(isReal)
export(isString)
export(is_NA)
export(is_option_symlink)
export(iterCount)
export(knit_ex)
export(latex_bibliography)
export(latex_preamble)
export(ldata)
export(libname)
export(library_project)
export(list.data)
export(list.libs)
export(listPackageOptions)
export(lmessage)
export(load_all_file)
export(load_project)
export(log_append)
export(lsilent)
export(lverbose)
export(makeFakeVignette)
export(makeUnitVignette)
export(make_vignette_auxfiles)
export(messagef)
export(mfrow)
export(mkoptions)
export(mrequire)
export(new2)
export(ns_get)
export(onLoad)
export(onUnload)
export(oneoffVariable)
export(option_symlink)
export(option_symlink_target)
export(orderVersion)
export(packageData)
export(packageEnv)
export(packageExtra)
export(packageExtraHandler)
export(packageExtraRunner)
export(packageOptions)
export(packagePath)
export(packageReference)
export(packageReferenceFile)
export(packageRegistries)
export(packageRegistry)
export(packageTestEnv)
export(package_bibliography)
export(parsePackageCitation)
export(pkgreg_fetch)
export(pkgreg_remove)
export(postponeAction)
export(qlibrary)
export(qrequire)
export(quickinstall)
export(read.yaml_section)
export(regfetch)
export(render_notes)
export(reorder_columns)
export(require.quiet)
export(requirePackage)
export(requireRUnit)
export(rnw)
export(rnwChildren)
export(rnwCompiler)
export(rnwDriver)
export(rnwIncludes)
export(rnwWrapper)
export(runPostponedAction)
export(runVignette)
export(sVariable)
export(setBiocMirror)
export(setCRANMirror)
export(setPackageExtra)
export(setPackageExtraHandler)
export(setPackageRegistry)
export(setPackageRegistryEntry)
export(setupPackageOptions)
export(simpleRegistry)
export(sortVersion)
export(source_files)
export(str_bs)
export(str_class)
export(str_desc)
export(str_diff)
export(str_dim)
export(str_fun)
export(str_hash)
export(str_md5sum)
export(str_ns)
export(str_out)
export(str_pkg)
export(sys.function_digest)
export(sys.function_frame)
export(sys.function_nframe)
export(sys.source_file)
export(testRversion)
export(topns)
export(topns_name)
export(unit.test)
export(unlist_)
export(unlist_with_sep)
export(userData)
export(userIs)
export(using_collate)
export(using_dir)
export(using_env)
export(using_envvar)
export(using_lib)
export(using_libpaths)
export(using_locale)
export(using_options)
export(using_par)
export(using_path)
export(utest)
export(utestFramework)
export(utestPath)
export(vignetteMakefile)
export(vmessage)
export(winbuild)
export(wnote)
export(write.pkgbib)
export(writeUnitVignette)
if (getRversion() < "3.5.0") export(isFALSE)
import(codetools)
import(digest)
import(grDevices)
import(graphics)
import(methods)
import(registry)
import(stats)
import(stringr)
import(utils)
importFrom(assertthat,assert_that)
importFrom(assertthat,is.number)
importFrom(assertthat,is.scalar)
importFrom(tools,Rd_db)
importFrom(tools,md5sum)
importFrom(withr,with_options)
importFrom(xtable,xtable)
