
\documentclass[10pt]{article}
%\VignetteDepends{knitr}
%\VignetteIndexEntry{pkgmaker-unitTests}
%\VignetteCompiler{knitr}
%\VignetteEngine{knitr::knitr}
\usepackage{vmargin}
\setmargrb{0.75in}{0.75in}{0.75in}{0.75in}

<<setup, include=FALSE>>=
pkg <- 'pkgmaker'
require( pkg, character.only=TRUE )
prettyVersion <- packageDescription(pkg)$Version
prettyDate <- format(Sys.Date(), '%B %e, %Y')
authors <- packageDescription(pkg)$Author
@

\usepackage[colorlinks]{hyperref}
\author{\Sexpr{authors}}
\title{\texttt{\Sexpr{pkg}}: Unit testing results\footnote{Vignette computed  on Wed May 14 13:48:07 2014}}
\date{\texttt{\Sexpr{pkg}} version \Sexpr{prettyVersion} as of \Sexpr{prettyDate}}
\begin{document}
\maketitle

\section{Details}
\begin{verbatim}

RUNIT TEST PROTOCOL -- Wed May 14 13:48:07 2014 
*********************************************** 
Number of test functions: 6 
Number of errors: 0 
Number of failures: 0 

 
1 Test Suite : 
package:pkgmaker - 6 test functions, 0 errors, 0 failures



Details 
*************************** 
Test Suite: package:pkgmaker 
Test function regexp: ^test. 
Test file regexp: ^runit.*.[rR]$ 
Involved directory: 
/tmp/Rpkglib_74c07de1ec8/pkgmaker/tests 
--------------------------- 
Test file: /tmp/Rpkglib_74c07de1ec8/pkgmaker/tests/runit.options.r 
test.option_link: (12 checks) ... OK (0.01 seconds)
test.resetOptions: (9 checks) ... OK (0 seconds)
--------------------------- 
Test file: /tmp/Rpkglib_74c07de1ec8/pkgmaker/tests/runit.utils.r 
test.errorCheck: (8 checks) ... OK (0 seconds)
test.ExposeAttribute: (20 checks) ... OK (0.01 seconds)
test.str_bs: (7 checks) ... OK (0 seconds)
test.Sys.getenv_value: (6 checks) ... OK (0 seconds)

Total execution time
***************************
   user  system elapsed 
  0.187   0.000   0.187 

\end{verbatim}

\section*{Session Information}
\begin{itemize}\raggedright
  \item R version 3.1.0 (2014-04-10), \verb|x86_64-pc-linux-gnu|
  \item Locale: \verb|LC_CTYPE=en_US.UTF-8|, \verb|LC_NUMERIC=C|, \verb|LC_TIME=en_US.UTF-8|, \verb|LC_COLLATE=en_US.UTF-8|, \verb|LC_MONETARY=en_US.UTF-8|, \verb|LC_MESSAGES=en_US.UTF-8|, \verb|LC_PAPER=en_US.UTF-8|, \verb|LC_NAME=C|, \verb|LC_ADDRESS=C|, \verb|LC_TELEPHONE=C|, \verb|LC_MEASUREMENT=en_US.UTF-8|, \verb|LC_IDENTIFICATION=C|
  \item Base packages: base, datasets, graphics, grDevices, methods,
    stats, utils
  \item Other packages: pkgmaker~0.22, registry~0.2, RUnit~0.4.26,
    stringr~0.6.2
  \item Loaded via a namespace (and not attached): codetools~0.2-8,
    digest~0.6.4, tools~3.1.0, xtable~1.7-3
\end{itemize}

\end{document}

