% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\name{onLoad}
\alias{onLoad}
\alias{onUnload}
\title{Default Load/Unload Functions}
\usage{
onLoad(libname = NULL, pkgname, chname = packageName())

onUnload(libpath)
}
\arguments{
\item{libname}{a character string giving the library directory where
    the package defining the namespace was found.}

\item{pkgname}{a character string giving the name of the package.}

\item{chname}{a character string naming a DLL (also known as a dynamic
    shared object or library) to load.}

\item{libpath}{a character string giving the complete path to the package.}
}
\value{
Returns nothing, used only for their side-effects.
}
\description{
Default Load/Unload Functions
}
\examples{

# in a package namespace:
.onLoad <- function(libname=NULL, pkgname){

	pkgmaker::onLoad(libname, pkgname)

}

# in a package namespace:
.onUnload <- function(libpath){

	pkgmaker::onUnload(libpath)

}
}
