% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{Rversion}
\alias{Rversion}
\title{Complete R version}
\usage{
Rversion()
}
\value{
A character string.
}
\description{
Returns the complete R version, e.g. 2.15.0
}
\examples{
Rversion()

}
